/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.fachstelle;

import java.util.List;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpHeaders;
import org.springframework.security.authentication.AuthenticationTrustResolver;
import org.springframework.security.authentication.AuthenticationTrustResolverImpl;
import org.springframework.security.config.Customizer;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configurers.oauth2.server.resource.OAuth2ResourceServerConfigurer;
import org.springframework.security.saml2.provider.service.registration.RelyingPartyRegistrationRepository;
import org.springframework.security.saml2.provider.service.web.DefaultRelyingPartyRegistrationResolver;
import org.springframework.security.saml2.provider.service.web.authentication.OpenSaml4AuthenticationRequestResolver;
import org.springframework.security.saml2.provider.service.web.authentication.Saml2AuthenticationRequestResolver;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.web.cors.CorsConfiguration;
import org.springframework.web.cors.UrlBasedCorsConfigurationSource;
import org.springframework.web.filter.CorsFilter;

import de.ozgcloud.fachstelle.security.FachstelleLogoutSuccessHandler;
import de.ozgcloud.fachstelle.security.InMemoryUserDetailService;
import de.ozgcloud.fachstelle.security.SecurityProvider;
import de.ozgcloud.fachstelle.security.UrlAuthenticationSuccessHandler;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Configuration
@Log4j2
@EnableWebSecurity
@RequiredArgsConstructor
public class SecurityConfiguration {

	private final InMemoryUserDetailService userDetailsService;
	private final UrlAuthenticationSuccessHandler urlAuthenticationSuccessHandler;
	private final FachstellenProperties properties;
	private final SpringJwtProperties springJwtProperties;

	@Bean
	public SecurityProvider securityProvider() {
		return new SecurityProvider();
	}

	@Bean
	public AuthenticationTrustResolver authenticationTrustResolver() {
		return new AuthenticationTrustResolverImpl();
	}

	@Bean
	public Saml2AuthenticationRequestResolver authenticationRequestResolver(RelyingPartyRegistrationRepository registrations) {
		var registrationResolver = new DefaultRelyingPartyRegistrationResolver(registrations);
		var authenticationRequestResolver = new OpenSaml4AuthenticationRequestResolver(registrationResolver);
		authenticationRequestResolver.setAuthnRequestCustomizer(context -> context.getAuthnRequest().setForceAuthn(true));

		return authenticationRequestResolver;
	}

	@Bean
	public SecurityFilterChain filterChain(HttpSecurity http) throws Exception {
		http
				.authorizeHttpRequests(authorize -> authorize
						.requestMatchers("/registrierung", "/static/**", "/webjars/**", "/actuator/**", "/error", "/fonts/**", "/webjars/**",
								"/api/environment")
						.permitAll()
						.requestMatchers("/api/**", "/preregister", "/register", "/success").authenticated()
						.anyRequest().denyAll());

		http.oauth2ResourceServer(this::setOAuth2ResourceServer);
		http.saml2Login(samlLogin -> samlLogin.successHandler(urlAuthenticationSuccessHandler))
				.saml2Logout(Customizer.withDefaults())
				.logout(logoutConfigurer -> logoutConfigurer.logoutSuccessHandler(getLogoutSuccessHandler()));

		return http.build();
	}

	private void setOAuth2ResourceServer(OAuth2ResourceServerConfigurer<HttpSecurity> configurer) {
		configurer.jwt().jwkSetUri(springJwtProperties.getJwkSetUri());
	}

	FachstelleLogoutSuccessHandler getLogoutSuccessHandler() {
		var handler = new FachstelleLogoutSuccessHandler(userDetailsService);
		handler.setDefaultTargetUrl(properties.getLogoutSuccessUrl());

		return handler;
	}

	@Bean
	public CorsFilter corsFilter() {
		var source = new UrlBasedCorsConfigurationSource();
		var config = new CorsConfiguration();
		config.setAllowCredentials(false);
		config.setAllowedOrigins(List.of(properties.getCors()));
		config.setAllowedMethods(List.of("POST", "OPTIONS", "GET", "HEAD"));
		config.setAllowedHeaders(
				List.of(HttpHeaders.ORIGIN, HttpHeaders.CONTENT_TYPE, HttpHeaders.ACCEPT, HttpHeaders.AUTHORIZATION, "X-Requested-With", "X-Client"));
		config.setExposedHeaders(List.of(HttpHeaders.LOCATION, HttpHeaders.CONTENT_DISPOSITION));
		source.registerCorsConfiguration("/**", config);

		return new CorsFilter(source);
	}
}
