/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.fachstelle.attachment;

import java.io.IOException;
import java.io.OutputStream;
import java.nio.charset.StandardCharsets;
import java.util.stream.Stream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.mvc.method.annotation.StreamingResponseBody;

import de.ozgcloud.apilib.file.OzgCloudFileId;
import de.ozgcloud.common.errorhandling.TechnicalException;
import de.ozgcloud.fachstelle.vorgang.Vorgang;
import lombok.RequiredArgsConstructor;

@RestController
@RequestMapping(AttachmentController.PATH)
@RequiredArgsConstructor
public class AttachmentController {

	static final String PATH = "/api/attachments"; // NOSONAR
	private static final String PARAM_EINGANG_ID = "eingangId";

	private final BinaryFileModelAssembler modelAssembler;

	@GetMapping(params = PARAM_EINGANG_ID)
	public CollectionModel<EntityModel<OzgFile>> getAllByEingang(@RequestParam String eingangId) {
		var attachments = Stream.of(getMockedAttachment(eingangId));

		return modelAssembler.toCollectionModel(attachments);
	}

	private OzgFile getMockedAttachment(String id) {
		return OzgFile.builder()
		  .id(OzgCloudFileId.from(id))
		  .name("TestFileName")
		  .size(1024)
		  .contentType("text/plain")
		  .build();
	}

	@RestController
	@RequestMapping(AttachmentsByVorgangController.PATH)
	public static class AttachmentsByVorgangController {

		static final String PATH = "/api/vorgang/{vorgangId}/attachments"; // NOSONAR

		@GetMapping(params = PARAM_EINGANG_ID, produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
		public ResponseEntity<StreamingResponseBody> download(@PathVariable String vorgangId, @RequestParam String eingangId) {
			return ResponseEntity.ok()
			  .header(HttpHeaders.CONTENT_DISPOSITION, String.format("attachment; filename=%s", buildZipName()))
			  .contentType(MediaType.APPLICATION_OCTET_STREAM)
			  .body(out -> createZipFile(out, eingangId));
		}

		String buildZipName() {
			var vorgang = Vorgang.builder().nummer("TestVorgangNummer").build(); // TODO: Vorgang mit vorgangId und address laden

			return vorgang.nummer() + "_Anhaenge.zip";
		}

		void createZipFile(OutputStream out, String eingangId) {
			try (ZipOutputStream zipOutputStream = new ZipOutputStream(out)) {
				var mockFile = new ZipEntry(String.format("%s.txt", eingangId));

				zipOutputStream.putNextEntry(mockFile);
				zipOutputStream.write("TestContent".getBytes(StandardCharsets.UTF_8));
				zipOutputStream.closeEntry();
			} catch (IOException e) {
				throw new TechnicalException("Error creating zip file", e);
			}
		}

	}

}