/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.fachstelle.common;

import java.util.Arrays;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Objects;
import java.util.function.BooleanSupplier;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.function.UnaryOperator;
import java.util.stream.Collectors;

import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;

import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
public class ModelBuilder<T> {

	private final T entity;
	private final EntityModel<T> model;
	private final List<Link> links = new LinkedList<>();
	private final List<Function<EntityModel<T>, EntityModel<T>>> mapper = new LinkedList<>();

	private ModelBuilder(T entity) {
		this.entity = entity;
		this.model = null;
	}

	private ModelBuilder(EntityModel<T> model) {
		this.entity = null;
		this.model = model;
	}

	public static <T> ModelBuilder<T> fromEntity(T entity) {
		return new ModelBuilder<>(entity);
	}

	public static <T> ModelBuilder<T> fromModel(EntityModel<T> model) {
		return new ModelBuilder<>(model);
	}

	public ModelBuilder<T> addLink(Link link) {
		links.add(link);

		return this;
	}

	public ConditionalLinkAdder ifMatch(Predicate<T> predicate) {
		return new ConditionalLinkAdder(predicate.test(Objects.isNull(entity) ? model.getContent() : entity));
	}

	public ConditionalLinkAdder ifMatch(BooleanSupplier guard) {
		return new ConditionalLinkAdder(guard.getAsBoolean());
	}

	public ModelBuilder<T> map(UnaryOperator<EntityModel<T>> mapper) {
		this.mapper.add(mapper);

		return this;
	}

	public EntityModel<T> buildModel() {
		var filteredLinks = links.stream().filter(Objects::nonNull).collect(Collectors.toSet());

		EntityModel<T> buildedModel = Objects.isNull(model) ? EntityModel.of(entity) : model;
		buildedModel = buildedModel.add(filteredLinks);

		return applyMapper(buildedModel);
	}

	private EntityModel<T> applyMapper(EntityModel<T> resource) {
		Iterator<Function<EntityModel<T>, EntityModel<T>>> i = mapper.iterator();
		EntityModel<T> result = resource;

		while (i.hasNext()) {
			result = i.next().apply(result);
		}

		return result;
	}

	@RequiredArgsConstructor
	public class ConditionalLinkAdder {

		public final boolean conditionFulfilled;

		public ModelBuilder<T> addLink(Link link) {
			if (conditionFulfilled) {
				links.add(link);
			}
			return ModelBuilder.this;
		}

		public ModelBuilder<T> addLinks(Link... links) {
			if (conditionFulfilled) {
				ModelBuilder.this.links.addAll(Arrays.asList(links));
			}

			return ModelBuilder.this;
		}

	}

}