/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.fachstelle.kommentar;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.function.Predicate;
import java.util.stream.Stream;

import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.server.RepresentationModelAssembler;
import org.springframework.stereotype.Component;

import de.ozgcloud.fachstelle.attachment.BinaryFileController;
import de.ozgcloud.fachstelle.common.CollectionModelBuilder;
import de.ozgcloud.fachstelle.common.ModelBuilder;
import de.ozgcloud.fachstelle.vorgang.Vorgang;
import de.ozgcloud.fachstelle.vorgang.VorgangStatus;

@Component
class KommentarModelAssembler implements RepresentationModelAssembler<Kommentar, EntityModel<Kommentar>> {

	static final String REL_CREATE = "create-kommentar";
	static final String REL_EDIT = "edit";
	static final String REL_ATTACHMENTS = "attachments";
	static final String REL_UPLOAD_FILE = "uploadFile";

	private static final Predicate<Kommentar> HAS_ATTACHMENTS = kommentar -> !kommentar.attachments().isEmpty();

	@Override
	public EntityModel<Kommentar> toModel(Kommentar kommentar) {
		var vorgang = Vorgang.builder().status(VorgangStatus.NEU).build(); // TODO: Vorgang mit vorgangId und address laden

		return buildModel(kommentar, vorgang);
	}

	EntityModel<Kommentar> buildModel(Kommentar kommentar, Vorgang vorgang) {
		var selfLink = linkTo(KommentarController.class).slash(kommentar.id());
		var commandLink = linkTo(methodOn(KommentarCommandController.class).editKommentar(kommentar.id(), kommentar.version()));
		var attachmentsLink = linkTo(methodOn(KommentarController.class).getAttachments(kommentar.id()));

		return ModelBuilder.fromEntity(kommentar)
		  .addLink(selfLink.withSelfRel())
		  .ifMatch(() -> isEditable(vorgang))
		  .addLink(commandLink.withRel(REL_EDIT))
		  .ifMatch(HAS_ATTACHMENTS)
		  .addLink(attachmentsLink.withRel(REL_ATTACHMENTS))
		  .buildModel();
	}

	public CollectionModel<EntityModel<Kommentar>> toCollectionModel(Stream<Kommentar> entities, String vorgangId) {
		var vorgang = Vorgang.builder().status(VorgangStatus.NEU).build(); // TODO: Vorgang mit vorgangId und address laden

		return CollectionModelBuilder.fromEntities(entities.map(kommentar -> buildModel(kommentar, vorgang)))
		  .addLink(linkTo(KommentarController.class).withSelfRel())
		  .addLink(linkTo(BinaryFileController.class).slash(vorgangId).slash("kommentarAttachment").slash("file").withRel(REL_UPLOAD_FILE))
		  .ifMatch(kommentareModel -> isEditable(vorgang))
		  .addLink(
			linkTo(methodOn(KommentarCommandController.KommentarCommandByVorgangController.class).createKommentar(vorgangId)).withRel(REL_CREATE))
		  .buildModel();
	}

	private boolean isEditable(Vorgang vorgang) {
		return vorgang.status() != VorgangStatus.ZU_LOESCHEN;
	}

}