/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.fachstelle.registration;

import java.time.Instant;
import java.time.temporal.ChronoUnit;
import java.util.NoSuchElementException;
import java.util.UUID;

import jakarta.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;
import org.springframework.security.core.Authentication;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PostMapping;

import de.ozgcloud.fachstelle.FachstellenProperties;
import de.ozgcloud.fachstelle.security.InMemoryUserDetailService;
import de.ozgcloud.fachstelle.security.User;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Controller
@RequiredArgsConstructor
public class FachstelleRegistrationController {

	public static final String OK = "ok";
	private final InMemoryUserDetailService userDetailsService;
	private final FachstellenProperties properties;
	private final FachstelleRegistrationService registrationService;

	@GetMapping(value = "/registrierung", produces = "text/html; charset=utf-8")
	public String startRegistration(final Model model) {
		model.addAttribute("loginUrl", properties.getLoginRedirectUrl());
		return "index";
	}

	@GetMapping(value = "/preregister", produces = "text/html; charset=utf-8")
	public String preregister(final Model model, Authentication auth) {
		var user = getUser(auth);
		addRegistrationKey(user);

		model.addAttribute("organizationName", user.getCompanyName());
		model.addAttribute("organizationLegalForm", user.getLegalFormText());
		model.addAttribute("organizationRegisterType", user.getRegisterType());
		model.addAttribute("organizationRegisterNumber", user.getRegisterNumber());
		model.addAttribute("organizationEmail", user.getEmailAddress());
		model.addAttribute("organizationAddress", user.getAddress());

		var registration = new Registration(user.getRegistrationKey());
		model.addAttribute("registration", registration);

		return "preregister";
	}

	void addRegistrationKey(final User user) {
		user.setRegistrationKey(UUID.randomUUID().toString());
		user.setRegistrationKeyExpiresAt(Instant.now().plus(10, ChronoUnit.MINUTES));
	}

	@GetMapping(value = "/success", produces = "text/html; charset=utf-8")
	public String success(final Model model, Authentication auth, HttpServletRequest request) {
		var user = getUser(auth);
		model.addAttribute("name", user);
		userDetailsService.logout(user);
		request.getSession().invalidate();

		return "success";
	}

	@PostMapping("/register")
	public String register(@ModelAttribute Registration registration, Model model, final Authentication auth) {
		var user = getUser(auth);

		var res = canRegister(user, registration.key());
		if (OK.equals(res) && registrationService.register(user)) {
			return "success";
		} else {
			model.addAttribute("errorMessageKey", res);
			return "error";
		}
	}

	private User getUser(Authentication auth) {
		try {
			return userDetailsService.loadUserByUsername(auth.getName());
		} catch (NoSuchElementException | NullPointerException e) {
			LOG.error("Error loading user.", e);
			throw new SecurityException(e);
		}
	}

	String canRegister(User user, String registrationKey) {
		boolean regKeyMatch = StringUtils.isNotEmpty(user.getRegistrationKey()) && user.getRegistrationKey().equals(registrationKey);
		if (!regKeyMatch) {
			LOG.error("Registration key invalid");
			return "error_registration_key_does_not_exist";
		}
		boolean regKeyNotExpired = Instant.now().isBefore(user.getRegistrationKeyExpiresAt());
		if (!regKeyNotExpired) {
			LOG.error("Registration key expired");
			return "error_registration_key_expired";
		}

		return OK;
	}

}
