/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.fachstelle.security;

import java.time.Instant;
import java.time.temporal.ChronoUnit;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;

import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.stereotype.Service;

import lombok.extern.log4j.Log4j2;

@Service
@Log4j2
public class InMemoryUserDetailService implements UserDetailsService {

	private final ConcurrentHashMap<String, User> usersMap = new ConcurrentHashMap<>();

	public InMemoryUserDetailService() {
		LOG.debug("Init InMemoryUserDetailService");
	}

	public void addUser(final User user) {
		LOG.debug("Adding user: {}", user);
		user.setExpirationDate(Instant.now().plus(4, ChronoUnit.HOURS));

		usersMap.put(user.getId(), user);

		LOG.debug("users map: {}", usersMap);
	}

	void setUser(final User user) {
		usersMap.put(user.getId(), user);
	}

	User getUser(String id) {
		return usersMap.get(id);
	}

	@Override
	public User loadUserByUsername(String username) {
		var optionalEntry = usersMap.entrySet().stream().filter(entry -> username.equals(entry.getValue().getUsername())).findFirst();

		return optionalEntry.map(Map.Entry::getValue).orElseThrow();
	}

	@Scheduled(fixedRate = 4, timeUnit = TimeUnit.HOURS)
	void userCleanUp() {
		var expiredEntries = usersMap.entrySet().stream().filter(entry -> !entry.getValue().isCredentialsNonExpired()).toList();
		expiredEntries.forEach(expiredEntry -> logout(expiredEntry.getValue()));
	}

	public void logout(User user) {
		usersMap.remove(user.getId());
	}

}
