/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.fachstelle.security;

import java.io.IOException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.security.interfaces.RSAPrivateKey;
import java.util.Arrays;
import java.util.List;

import jakarta.annotation.PostConstruct;

import org.opensaml.core.config.InitializationException;
import org.opensaml.core.config.InitializationService;
import org.opensaml.saml.saml2.core.Assertion;
import org.opensaml.saml.saml2.core.Attribute;
import org.opensaml.saml.saml2.core.AttributeStatement;
import org.opensaml.saml.saml2.core.EncryptedAssertion;
import org.opensaml.saml.saml2.core.Response;
import org.opensaml.saml.saml2.encryption.Decrypter;
import org.opensaml.saml.saml2.encryption.EncryptedElementTypeEncryptedKeyResolver;
import org.opensaml.security.credential.CredentialSupport;
import org.opensaml.xmlsec.encryption.support.ChainingEncryptedKeyResolver;
import org.opensaml.xmlsec.encryption.support.EncryptedKeyResolver;
import org.opensaml.xmlsec.encryption.support.InlineEncryptedKeyResolver;
import org.opensaml.xmlsec.encryption.support.SimpleRetrievalMethodEncryptedKeyResolver;
import org.opensaml.xmlsec.keyinfo.impl.CollectionKeyInfoCredentialResolver;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.Resource;
import org.springframework.security.converter.RsaKeyConverters;
import org.springframework.security.saml2.Saml2Exception;
import org.springframework.security.saml2.core.Saml2X509Credential;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import lombok.Getter;
import lombok.Setter;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service
public class Saml2Decrypter {

	@Getter
	@Setter
	private Decrypter decrypter;

	@Value("${spring.security.saml2.relyingparty.registration.muk.decryption.credentials[0].private-key-location}")
	private Resource decryptionPrivateKeyLocation;

	@Value("${spring.security.saml2.relyingparty.registration.muk.decryption.credentials[0].certificate-location}")
	private Resource decryptionCertificateLocation;

	private static final EncryptedKeyResolver encryptedKeyResolver = new ChainingEncryptedKeyResolver(
	  Arrays.asList(new InlineEncryptedKeyResolver(), new EncryptedElementTypeEncryptedKeyResolver(),
		new SimpleRetrievalMethodEncryptedKeyResolver()));

	@PostConstruct
	void init() throws InitializationException {
		InitializationService.initialize();

		var decryptionX509Credential = getDecryptionCredential();
		var cred = CredentialSupport.getSimpleCredential(decryptionX509Credential.getCertificate(), decryptionX509Credential.getPrivateKey());

		var resolver = new CollectionKeyInfoCredentialResolver(List.of(cred));
		var setupDecrypter = new Decrypter(null, resolver, encryptedKeyResolver);
		setupDecrypter.setRootInNewDocument(true);

		decrypter = setupDecrypter;
	}

	Attribute getDecryptedAttribute(String samlResponse, String attributeName) {
		var parsedResponse = Saml2Parser.parse(samlResponse);

		decryptResponseElements(parsedResponse);

		return getAttributes(parsedResponse).stream().filter(attribute -> attributeName.equals(attribute.getName())).findFirst().orElseThrow();
	}

	private void decryptResponseElements(Response response) {
		response.getEncryptedAssertions().stream()
		  .map(this::decryptAssertion)
		  .forEach(assertion -> response.getAssertions().add(assertion));
	}

	private Assertion decryptAssertion(EncryptedAssertion assertion) {
		try {
			return decrypter.decrypt(assertion);
		} catch (Exception ex) {
			LOG.error("failed to decrypt assertion {}", assertion);
			throw new Saml2Exception(ex);
		}
	}

	private List<Attribute> getAttributes(Response response) {
		var attributeStatement = (AttributeStatement) response.getAssertions().getFirst().getStatements().get(1);

		return attributeStatement.getAttributes();
	}

	private Saml2X509Credential getDecryptionCredential() {
		var privateKey = readPrivateKey(decryptionPrivateKeyLocation);
		var certificate = readCertificateFromResource(decryptionCertificateLocation);

		return new Saml2X509Credential(privateKey, certificate, Saml2X509Credential.Saml2X509CredentialType.DECRYPTION);
	}

	private RSAPrivateKey readPrivateKey(Resource location) {
		Assert.state(location != null, "No private key location specified");
		Assert.state(location.exists(), () -> "Private key location '" + location + "' does not exist");

		try (var inputStream = location.getInputStream()) {
			return RsaKeyConverters.pkcs8().convert(inputStream);
		} catch (IOException e) {
			throw new IllegalArgumentException(e);
		}
	}

	private X509Certificate readCertificateFromResource(Resource location) {
		Assert.state(location != null, "No certificate location specified");
		Assert.state(location.exists(), () -> "Certificate  location '" + location + "' does not exist");

		try (var inputStream = location.getInputStream()) {
			return (X509Certificate) CertificateFactory.getInstance("X.509").generateCertificate(inputStream);
		} catch (IOException | CertificateException e) {
			throw new IllegalArgumentException(e);
		}
	}

}