/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.fachstelle.security;

import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.Objects;

import org.apache.commons.lang3.ArrayUtils;
import org.opensaml.core.xml.XMLObject;
import org.springframework.security.saml2.Saml2Exception;
import org.springframework.security.saml2.provider.service.authentication.DefaultSaml2AuthenticatedPrincipal;
import org.springframework.stereotype.Component;

import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Component
@RequiredArgsConstructor
class UserAttributeProvider {

	static final String SAML_XML_STRASSE_NODE_NAME = "Strasse";
	static final String SAML_XML_HAUSNUMMER_NODE_NAME = "Hausnummer";
	static final String SAML_XML_PLZ_NODE_NAME = "PLZ";
	static final String SAML_XML_ORT_NODE_NAME = "Ort";
	static final String SAML_XML_LAND_NODE_NAME = "Land";

	static final String MUK_FIRMENNAME_KEY = "Firmenname";
	static final String MUK_RECHTSFORM_KEY = "Rechtsform";
	static final String MUK_RECHTSFORM_TEXT_KEY = "RechtsformText";
	static final String MUK_REGISTERNUMMER_KEY = "Registernummer";
	static final String MUK_REGISTERART_KEY = "Registerart";
	static final String MUK_EMAIL_ADRESSE_KEY = "EMailAdresse";
	static final String MUK_ADRESSE_KEY = "Unternehmensanschrift";
	static final String MUK_VERTRAUENSNIVEAU_KEY = "ElsterVertrauensniveauAuthentifizierung";
	private static final String[] MUK_KNOWN_ATTRIBUTES = new String[] { MUK_FIRMENNAME_KEY, MUK_RECHTSFORM_KEY, MUK_RECHTSFORM_TEXT_KEY,
	  MUK_REGISTERNUMMER_KEY, MUK_REGISTERART_KEY, MUK_EMAIL_ADRESSE_KEY, MUK_ADRESSE_KEY, MUK_VERTRAUENSNIVEAU_KEY };

	private final Saml2Decrypter saml2Decrypter;

	String getCompanyName(DefaultSaml2AuthenticatedPrincipal principal) {
		return principal.getFirstAttribute(MUK_FIRMENNAME_KEY);
	}

	String getLegalForm(DefaultSaml2AuthenticatedPrincipal principal) {
		return principal.getFirstAttribute(MUK_RECHTSFORM_KEY);
	}

	String getLegalFormText(DefaultSaml2AuthenticatedPrincipal principal) {
		return principal.getFirstAttribute(MUK_RECHTSFORM_TEXT_KEY);
	}

	String getRegisterNumber(DefaultSaml2AuthenticatedPrincipal principal) {
		return principal.getFirstAttribute(MUK_REGISTERNUMMER_KEY);
	}

	String getRegisterType(DefaultSaml2AuthenticatedPrincipal principal) {
		return principal.getFirstAttribute(MUK_REGISTERART_KEY);
	}

	String getEmailAddress(DefaultSaml2AuthenticatedPrincipal principal) {
		return principal.getFirstAttribute(MUK_EMAIL_ADRESSE_KEY);
	}

	String getTrustLevel(DefaultSaml2AuthenticatedPrincipal principal) {
		return principal.getFirstAttribute(MUK_VERTRAUENSNIVEAU_KEY);
	}

	String getAddress(String samlResponse) {
		try {
			var addressNode = saml2Decrypter.getDecryptedAttribute(samlResponse, MUK_ADRESSE_KEY);
			var addressPartNodes = addressNode.getAttributeValues().getFirst().getOrderedChildren();

			if (addressPartNodes != null && !addressPartNodes.isEmpty()) {
				return getAddressByXMLNodes(addressPartNodes);
			}
		} catch (IllegalArgumentException | Saml2Exception | NoSuchElementException e) {
			LOG.error("Failed parsing company address from SamlResponse: {}", samlResponse);
		}

		return null;
	}

	private static String getAddressByXMLNodes(final List<XMLObject> addressPartNodes) {
		var addressBuilder = new StringBuilder();

		for (XMLObject node : addressPartNodes) {
			var nodeName = node.getElementQName().getLocalPart();
			var textContent = Objects.requireNonNull(node.getDOM()).getTextContent().trim();

			if (SAML_XML_STRASSE_NODE_NAME.equals(nodeName) || SAML_XML_PLZ_NODE_NAME.equals(nodeName)) {
				addressBuilder.append(textContent).append(" ");
			} else if (SAML_XML_HAUSNUMMER_NODE_NAME.equals(nodeName) || SAML_XML_ORT_NODE_NAME.equals(nodeName)) {
				addressBuilder.append(textContent).append(", ");
			} else if (SAML_XML_LAND_NODE_NAME.equals(nodeName)) {
				addressBuilder.append(textContent);
			}
		}

		return addressBuilder.toString().trim();
	}

	List<Map.Entry<String, List<Object>>> getUnknownAttributes(DefaultSaml2AuthenticatedPrincipal principal) {
		return principal.getAttributes().entrySet().stream().filter(entry -> !ArrayUtils.contains(MUK_KNOWN_ATTRIBUTES, entry.getKey())).toList();
	}

}
