/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.fachstelle;

import static de.ozgcloud.fachstelle.RootModelAssembler.*;
import static de.ozgcloud.fachstelle.RootTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.Objects;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;

import de.ozgcloud.fachstelle.vorgang.VorgangController;

@ExtendWith(MockitoExtension.class)
class RootControllerTest {

	@Nested
	class TestWithBuildProperties {

		@Spy
		@InjectMocks
		private RootController rootController;

		@Mock
		private RootModelAssembler rootModelAssembler;

		@BeforeEach
		void setUp() {
			when(rootModelAssembler.toModel(any(Root.class))).thenReturn(
			  EntityModel.of(
				RootTestFactory.create()
			  ).add(linkTo(RootController.class).withSelfRel(), linkTo(methodOn(VorgangController.class).getAll()).withRel(ALL_VORGAENGE_REL)));
		}

		@Test
		void shouldHaveRootEntityModel() {
			assertThat(rootController.getRoot()).isNotNull();
		}

		@Test
		void shouldHaveSelfLink() {
			var model = rootController.getRoot();

			assertThat(model.getLink("self")).isPresent().get().extracting(Link::getHref).isEqualTo("/api");
		}

		@Test
		void shouldHaveVorgaengeAllLink() {
			var model = rootController.getRoot();

			assertThat(model.getLink(ALL_VORGAENGE_REL)).isPresent().get().extracting(Link::getHref).isEqualTo("/api/vorgangs");
		}

		@Test
		void shouldHaveRoot() {
			assertThat(rootController.getRoot().getContent()).isNotNull();
		}

		@Test
		void shouldHaveBuildVersion() {
			var root = Objects.requireNonNull(rootController.getRoot().getContent());

			assertThat(root.version()).isEqualTo(VERSION);
		}

		@Test
		void shouldHaveBuildTime() {
			var root = Objects.requireNonNull(rootController.getRoot().getContent());

			assertThat(root.buildTime()).isEqualTo(RootTestFactory.BUILD_TIME);
		}

		@Test
		void shouldHaveJavaVersion() {
			var root = Objects.requireNonNull(rootController.getRoot().getContent());

			assertThat(root.javaVersion()).isEqualTo(JAVA_VERSION);
		}

		@Test
		void shouldHaveProduction() {
			var root = Objects.requireNonNull(rootController.getRoot().getContent());

			assertThat(root.production()).isTrue();
		}

	}

	@Nested
	class TestWithoutBuildProperties {

		private final RootController rootController = new RootController(null, new RootModelAssembler());

		@Test
		void shouldHaveRootEntityModel() {
			assertThat(rootController.getRoot()).isNotNull();
		}

		@Test
		void shouldHaveRoot() {
			assertThat(rootController.getRoot().getContent()).isNotNull();
		}

		@Test
		void shouldHaveBuildVersionDefault() {
			var root = Objects.requireNonNull(rootController.getRoot().getContent());

			assertThat(root.version()).isEqualTo("--");
		}

		@Test
		void shouldHaveBuildTimeDefault() {
			var root = Objects.requireNonNull(rootController.getRoot().getContent());

			assertThat(root.buildTime()).isNull();
		}

		@Test
		void shouldHaveJavaVersion() {
			var root = Objects.requireNonNull(rootController.getRoot().getContent());

			assertThat(root.javaVersion()).isNotEmpty();
		}

		@Test
		void shouldHaveProduction() {
			var root = Objects.requireNonNull(rootController.getRoot().getContent());

			assertThat(root.production()).isTrue();
		}

	}

}