/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.fachstelle;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.saml2.provider.service.registration.RelyingPartyRegistrationRepository;
import org.springframework.security.saml2.provider.service.web.authentication.Saml2AuthenticationRequestResolver;

import de.ozgcloud.fachstelle.security.FachstelleLogoutSuccessHandler;
import de.ozgcloud.fachstelle.security.InMemoryUserDetailService;
import de.ozgcloud.fachstelle.security.UrlAuthenticationSuccessHandler;

@ExtendWith(MockitoExtension.class)
class SecurityConfigurationTest {

	@InjectMocks
	private SecurityConfiguration securityConfiguration;

	@Mock
	private InMemoryUserDetailService userDetailsService;
	@Mock
	private UrlAuthenticationSuccessHandler urlAuthenticationSuccessHandler;

	@Mock
	private FachstellenProperties fachstellenProperties;
	@Mock
	private SpringJwtProperties springJwtProperties;

	@Test
	void shouldCreateSecurityProvider() {
		var securityProvider = securityConfiguration.securityProvider();

		assertThat(securityProvider).isNotNull();
	}

	@Test
	void shouldCreateAuthenticationTrustResolver() {
		var authenticationTrustResolver = securityConfiguration.authenticationTrustResolver();

		assertThat(authenticationTrustResolver).isNotNull();
	}

	@Nested
	class TestHttpSecurity {

		private HttpSecurity httpSecurity;

		@BeforeEach
		void init() throws Exception {
			httpSecurity = mock(HttpSecurity.class);
			when(httpSecurity.authorizeHttpRequests(any())).thenReturn(httpSecurity);
			when(httpSecurity.saml2Login(any())).thenReturn(httpSecurity);
			when(httpSecurity.saml2Logout(any())).thenReturn(httpSecurity);
		}

		@Test
		void shouldSetupFilterChainAuthorize() throws Exception {
			securityConfiguration.filterChain(httpSecurity);

			verify(httpSecurity).authorizeHttpRequests(any());
		}

		@Disabled
		@Test
		void shouldSetupFilterChainSaml2Login() throws Exception {
			securityConfiguration.filterChain(httpSecurity);

			verify(httpSecurity).saml2Login(any());
		}

		@Disabled
		@Test
		void shouldSetupFilterChainSaml2Logout() throws Exception {
			securityConfiguration.filterChain(httpSecurity);

			verify(httpSecurity).saml2Logout(any());
		}

	}

	@Nested
	class TestAuthenticationRequestResolver {

		@Mock
		private RelyingPartyRegistrationRepository registrations;

		@Test
		void shouldCreateSaml2AuthenticationRequestResolver() {
			var authResolver = securityConfiguration.authenticationRequestResolver(registrations);

			assertThat(authResolver).isInstanceOf(Saml2AuthenticationRequestResolver.class);
		}

	}

	@Nested
	class TestSamlFilterChain {

		@Mock
		HttpSecurity security;

		@BeforeEach
		void setUp() throws Exception {
			when(security.authorizeHttpRequests(any())).thenReturn(security);
			when(security.saml2Login(any())).thenReturn(security);
			when(security.saml2Logout(any())).thenReturn(security);
		}

		@Disabled
		@Test
		void shouldSetSamlLogin() throws Exception {
			securityConfiguration.filterChain(security);

			verify(security).saml2Login(any());
		}

		@Disabled
		@Test
		void shouldSetSamlLogout() throws Exception {
			securityConfiguration.filterChain(security);

			verify(security).saml2Logout(any());
		}

	}

	@Nested
	class TestLogoutHandler {

		@Test
		void shouldCreateLogoutHandler() {
			when(fachstellenProperties.getLogoutSuccessUrl()).thenReturn("http://localhost:8080/logout");

			var handler = securityConfiguration.getLogoutSuccessHandler();

			assertThat(handler).isInstanceOf(FachstelleLogoutSuccessHandler.class);
		}

	}

}