/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.fachstelle.attachment;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.hateoas.EntityModel;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import lombok.SneakyThrows;

@ExtendWith(MockitoExtension.class)
class BinaryFileControllerTest {

	@Spy
	@InjectMocks
	private BinaryFileController controller;

	@Mock
	private BinaryFileModelAssembler modelAssembler;

	private MockMvc mockMvc;

	@BeforeEach
	void init() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@Nested
	class TestGetFile {

		@SneakyThrows
		@Test
		void shouldReturnStatusOk() {
			performRequest().andExpect(status().isOk());
		}

		@Test
		@SneakyThrows
		void shouldCallAssembler() {
			performRequest();

			verify(modelAssembler).toModel(any());
		}

		@Test
		@SneakyThrows
		void shouldHaveFileId() {
			when(modelAssembler.toModel(any())).then(i -> EntityModel.of(i.getArgument(0)));

			performRequest().andExpect(jsonPath("$.id.value").value(OzgFileTestFactory.ID.toString()));
		}

		@Test
		@SneakyThrows
		void shouldHaveFileName() {
			when(modelAssembler.toModel(any())).then(i -> EntityModel.of(i.getArgument(0)));

			performRequest().andExpect(jsonPath("$.name").value("TestFileName"));
		}

		@Test
		@SneakyThrows
		void shouldHaveFileSize() {
			when(modelAssembler.toModel(any())).then(i -> EntityModel.of(i.getArgument(0)));

			performRequest().andExpect(jsonPath("$.size").value(1024));
		}

		@Test
		@SneakyThrows
		void shouldHaveFileContentType() {
			when(modelAssembler.toModel(any())).then(i -> EntityModel.of(i.getArgument(0)));

			performRequest().andExpect(jsonPath("$.contentType").value("text/plain"));
		}

		@SneakyThrows
		private ResultActions performRequest() {
			return mockMvc.perform(
			  get(BinaryFileController.PATH + "/" + OzgFileTestFactory.ID).header(HttpHeaders.ACCEPT, MediaType.APPLICATION_JSON));
		}

	}

	@Nested
	class TestUploadFile {

		@Test
		void shouldReturnStatusCreated() throws Exception {
			performRequest().andExpect(status().isCreated());
		}

		@Test
		void shouldHaveLocationHeader() throws Exception {
			performRequest().andExpect(header().exists(HttpHeaders.LOCATION));
		}

		@SneakyThrows
		private ResultActions performRequest() {
			return mockMvc.perform(asyncDispatch(mockMvc.perform(post(BinaryFileController.PATH + "/test/test/file")).andReturn()));
		}

	}

	@Nested
	class TestGetFileData {

		@SneakyThrows
		@Test
		void shouldReturnStatusOk() {
			performRequest().andExpect(status().isOk());
		}

		@SneakyThrows
		@Test
		void shouldHaveContentDispositionHeader() {
			performRequest().andExpect(header().string(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=TestFileName"));
		}

		@SneakyThrows
		@Test
		void shouldHaveContentType() {
			performRequest().andExpect(header().string(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_OCTET_STREAM_VALUE));
		}

		@SneakyThrows
		private ResultActions performRequest() {
			return mockMvc.perform(asyncDispatch(mockMvc.perform(
				get(BinaryFileController.PATH + "/test").header(HttpHeaders.ACCEPT, MediaType.APPLICATION_OCTET_STREAM_VALUE))
			  .andReturn()));
		}

	}

}