/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.fachstelle.common;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.hateoas.Link;

import de.ozgcloud.fachstelle.vorgang.VorgangTestFactory;

class ModelBuilderTest {

	private final String HREF = "http://test";
	private final String REL = "rel";

	@Nested
	class TestAddLink {

		@Test
		void shouldAddLink() {
			var model = ModelBuilder.fromEntity(VorgangTestFactory.create()).addLink(Link.of(HREF, REL)).buildModel();

			assertThat(model.getLinks()).hasSize(1).first().extracting(Link::getHref, link -> link.getRel().value()).containsExactly(HREF, REL);
		}

	}

	@Nested
	class TestIfMatch {

		@Nested
		class TestWithBooleanSupplier {

			@Test
			void shouldAddLink() {
				var model = ModelBuilder.fromEntity(VorgangTestFactory.create()).ifMatch(() -> true).addLink(Link.of(HREF, REL)).buildModel();

				assertThat(model.getLinks()).hasSize(1).first().extracting(Link::getHref, link -> link.getRel().value()).containsExactly(HREF, REL);
			}

			@Test
			void shouldNotAddLink() {
				var model = ModelBuilder.fromEntity(VorgangTestFactory.create()).ifMatch(() -> false).addLink(Link.of(HREF, REL)).buildModel();

				assertThat(model.getLinks()).isEmpty();
			}

		}

		@Nested
		class TestWithPredicate {

			@Test
			void shouldAddLink() {
				var model = ModelBuilder.fromEntity(VorgangTestFactory.create())
				  .ifMatch(vorgang -> true)
				  .addLink(Link.of(HREF, REL))
				  .buildModel();

				assertThat(model.getLinks()).hasSize(1).first().extracting(Link::getHref, link -> link.getRel().value()).containsExactly(HREF, REL);
			}

			@Test
			void shouldNotAddLink() {
				var model = ModelBuilder.fromEntity(VorgangTestFactory.create())
				  .ifMatch(vorgang -> false)
				  .addLink(Link.of(HREF, REL))
				  .buildModel();

				assertThat(model.getLinks()).isEmpty();
			}

		}

	}

	@Nested
	class TestBuildModel {

		@Test
		void shouldBuildModel() {
			var vorgang = VorgangTestFactory.create();
			var model = ModelBuilder.fromEntity(vorgang).buildModel();

			assertThat(model.getContent()).usingRecursiveComparison().isEqualTo(vorgang);
		}

	}

}