/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.fachstelle.common.errorhandling;

import static de.ozgcloud.fachstelle.common.errorhandling.ExceptionController.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import jakarta.servlet.http.HttpServletRequest;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.http.HttpStatus;
import org.springframework.web.context.request.WebRequest;

@ExtendWith(MockitoExtension.class)
public class ExceptionControllerTest {

	@InjectMocks
	private ExceptionController exceptionController;

	@Nested
	class TestHandleNotFoundException {

		@Mock
		HttpServletRequest request;

		@Mock
		NotFoundException exception;

		@Test
		void shouldCreateNotFoundView() {
			var modelAndView = exceptionController.handleNotFoundException(request, exception);

			assertThat(modelAndView).isNotNull();
		}

		@Test
		void shouldHaveMessage() {
			when(exception.getMessage()).thenReturn("message");

			var modelAndView = exceptionController.handleNotFoundException(request, exception);

			assertThat(modelAndView.getModel()).containsEntry("exception", "message");
		}

		@Test
		void shouldHaveUrl() {
			var val = new StringBuffer("http://localhost:8080/");
			when(request.getRequestURL()).thenReturn(val);

			var modelAndView = exceptionController.handleNotFoundException(request, exception);

			assertThat(modelAndView.getModel()).containsEntry("url", val);
		}

		@Test
		void shouldHaveViewName() {
			var modelAndView = exceptionController.handleNotFoundException(request, exception);

			assertThat(modelAndView.getViewName()).isEqualTo(DEFAULT_ERROR_VIEW);
		}

	}

	@Nested
	class TestHandleResourceNotFoundException {

		@Mock
		ResourceNotFoundException exception;

		@Test
		void shouldHaveResponse() {
			var response = exceptionController.handleResourceNotFoundException(exception, mock(WebRequest.class));

			assertThat(response).isNotNull();
		}

		@Test
		void shouldHaveNotFoundError() {
			var response = exceptionController.handleResourceNotFoundException(exception, mock(WebRequest.class));

			assertThat(response.getStatusCode()).isEqualTo(HttpStatus.NOT_FOUND);
		}

		@Test
		void shouldHaveMessage() {
			var response = exceptionController.handleResourceNotFoundException(exception, mock(WebRequest.class));

			assertThat(response.getBody()).isEqualTo("Resource not found");
		}

	}

	@Nested
	class TestHandleSamlTokenNotFoundException {

		@Mock
		SamlTokenNotFoundException exception;

		@Test
		void shouldHaveResponse() {
			var response = exceptionController.handleSamlTokenNotFoundException(exception, mock(WebRequest.class));

			assertThat(response).isNotNull();
		}

		@Test
		void shouldHaveUnauthorizedError() {
			var response = exceptionController.handleSamlTokenNotFoundException(exception, mock(WebRequest.class));

			assertThat(response.getStatusCode()).isEqualTo(HttpStatus.UNAUTHORIZED);
		}

		@Test
		void shouldHaveMessage() {
			var response = exceptionController.handleSamlTokenNotFoundException(exception, mock(WebRequest.class));

			assertThat(response.getBody()).isEqualTo("Saml token not found");
		}

	}

	@Nested
	class TestHandleRuntimeException {

		@Mock
		IllegalStateException exception;

		@Test
		void shouldHaveResponse() {
			var response = exceptionController.handleRuntimeException(exception, mock(WebRequest.class));

			assertThat(response).isNotNull();
		}

		@Test
		void shouldHaveInternalServerError() {
			var response = exceptionController.handleRuntimeException(exception, mock(WebRequest.class));

			assertThat(response.getStatusCode()).isEqualTo(HttpStatus.INTERNAL_SERVER_ERROR);
		}

		@Test
		void shouldHaveMessage() {
			var response = exceptionController.handleRuntimeException(exception, mock(WebRequest.class));

			assertThat(response.getBody()).isEqualTo("Unexpected error occurred");
		}

	}

}
