/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.fachstelle.kommentar;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.stream.Stream;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.IanaLinkRelations;
import org.springframework.hateoas.Link;

import de.ozgcloud.fachstelle.vorgang.Vorgang;
import de.ozgcloud.fachstelle.vorgang.VorgangStatus;
import de.ozgcloud.fachstelle.vorgang.VorgangTestFactory;

@ExtendWith(MockitoExtension.class)
class KommentarModelAssemblerTest {

	@Spy
	private KommentarModelAssembler modelAssembler;

	@Nested
	class TestToModel {

		@Test
		void shouldBuildModel() {
			var kommentar = KommentarTestFactory.create();

			modelAssembler.toModel(kommentar);

			verify(modelAssembler).buildModel(kommentar, Vorgang.builder().status(VorgangStatus.NEU).build());
		}

	}

	@Nested
	class TestBuildModel {

		private static final String KOMMENTAR_EDIT_PATH = KommentarCommandController.PATH.replace("{kommentarId}", KommentarTestFactory.ID)
		  .replace("{kommentarVersion}", String.valueOf(KommentarTestFactory.VERSION));

		@Test
		void shouldHaveSelfLink() {
			var link = buildModel().getLink(IanaLinkRelations.SELF);

			assertThat(link).isPresent().get().extracting(Link::getHref).isEqualTo(KommentarController.PATH + "/" + KommentarTestFactory.ID);
		}

		@Test
		void shouldHaveEditLink() {
			assertThat(buildModel().getLink(KommentarModelAssembler.REL_EDIT)).isPresent().get().extracting(Link::getHref)
			  .isEqualTo(KOMMENTAR_EDIT_PATH);
		}

		@Test
		void shouldNotHaveEditLink() {
			var vorgang = VorgangTestFactory.createBuilder().status(VorgangStatus.ZU_LOESCHEN).build();
			var model = modelAssembler.buildModel(KommentarTestFactory.create(), vorgang);

			assertThat(model.getLink(KommentarModelAssembler.REL_EDIT)).isEmpty();
		}

		@Test
		void shouldHaveAttachmentsLink() {
			var kommentar = KommentarTestFactory.create();
			var model = modelAssembler.buildModel(kommentar, VorgangTestFactory.create());

			assertThat(model.getLink(KommentarModelAssembler.REL_ATTACHMENTS)).isPresent().get().extracting(Link::getHref)
			  .isEqualTo(String.format("%s%s/attachments", KommentarController.PATH + "/", kommentar.id()));
		}

		@Test
		void shouldNotHaveAttachmentsLink() {
			var kommentar = KommentarTestFactory.createBuilder().attachments(Collections.emptyList()).build();
			var model = modelAssembler.buildModel(kommentar, VorgangTestFactory.create());

			assertThat(model.getLink(KommentarModelAssembler.REL_ATTACHMENTS)).isEmpty();
		}

		EntityModel<Kommentar> buildModel() {
			return modelAssembler.buildModel(KommentarTestFactory.create(), VorgangTestFactory.create());
		}

	}

	@Nested
	class TestToCollectionModel {

		@Test
		void shouldBuildModel() {
			var kommentar = KommentarTestFactory.create();

			modelAssembler.toCollectionModel(Stream.of(kommentar), VorgangTestFactory.ID);

			verify(modelAssembler).buildModel(kommentar, Vorgang.builder().status(VorgangStatus.NEU).build());
		}

		@Test
		void shouldHaveSelfLink() {
			var link = modelAssembler.toCollectionModel(Stream.of(KommentarTestFactory.create()), VorgangTestFactory.ID)
			  .getLink(IanaLinkRelations.SELF);

			assertThat(link).isPresent().get().extracting(Link::getHref).isEqualTo(KommentarController.PATH);
		}

		@Test
		void shouldHaveUploadFileLink() {
			var model = modelAssembler.toCollectionModel(Stream.of(KommentarTestFactory.create()), VorgangTestFactory.ID);
			var link = model.getLink(KommentarModelAssembler.REL_UPLOAD_FILE);

			assertThat(link).isPresent().get().extracting(Link::getHref)
			  .isEqualTo("/api/binaryFiles/" + VorgangTestFactory.ID + "/kommentarAttachment/file");
		}

		@Test
		void shouldHaveCreateKommentarLink() {
			var collectionModel = modelAssembler.toCollectionModel(Stream.of(KommentarTestFactory.create()), VorgangTestFactory.ID);
			var link = collectionModel.getLink(KommentarModelAssembler.REL_CREATE);

			assertThat(link).isPresent().get().extracting(Link::getHref).isEqualTo("/api/vorgangs/" + VorgangTestFactory.ID + "/kommentarCommands");
		}

	}

}