/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.fachstelle.registration;

import static de.ozgcloud.fachstelle.security.UserTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.time.Instant;
import java.time.temporal.ChronoUnit;

import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpSession;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.security.core.Authentication;
import org.springframework.ui.Model;

import de.ozgcloud.fachstelle.FachstellenProperties;
import de.ozgcloud.fachstelle.security.InMemoryUserDetailService;
import de.ozgcloud.fachstelle.security.User;

@ExtendWith(MockitoExtension.class)
class FachstelleRegistrationControllerTest {

	@InjectMocks
	@Spy
	private FachstelleRegistrationController fachstelleRegistrationController;
	@Mock
	private InMemoryUserDetailService userDetailService;
	@Mock
	private FachstellenProperties fachstellenProperties;
	@Mock
	private FachstelleRegistrationService registrationService;

	@Nested
	class TestStartRegistration {

		@Mock
		private Model model;

		@BeforeEach
		void setUp() {
			when(fachstellenProperties.getLoginRedirectUrl()).thenReturn("/login");
		}

		@Test
		void shouldCreateIndexPage() {
			var res = fachstelleRegistrationController.startRegistration(model);

			assertThat(res).isEqualTo("index");
		}

		@Test
		void shouldSetLoginUrl() {
			fachstelleRegistrationController.startRegistration(model);

			verify(model).addAttribute("loginUrl", "/login");
		}

	}

	@Nested
	class TestRegistrationPage {

		@Mock
		private Model model;

		@Mock
		private Authentication authentication;

		@BeforeEach
		void setUp() {
			when(authentication.getName()).thenReturn("user");
			when(userDetailService.loadUserByUsername("user")).thenReturn(create());
		}

		@Test
		void shouldCreateRegisterPage() {
			var res = fachstelleRegistrationController.preregister(model, authentication);

			assertThat(res).isEqualTo("preregister");
		}

		@Test
		void shouldSetOrganizationName() {
			fachstelleRegistrationController.preregister(model, authentication);

			verify(model).addAttribute("organizationName", COMPANY_NAME);
		}

		@Test
		void shouldSetOrganizationLegalForm() {
			fachstelleRegistrationController.preregister(model, authentication);

			verify(model).addAttribute("organizationLegalForm", LEGAL_FORM_TEXT);
		}

		@Test
		void shouldSetOrganizationRegisterType() {
			fachstelleRegistrationController.preregister(model, authentication);

			verify(model).addAttribute("organizationRegisterType", REGISTER_TYPE);
		}

		@Test
		void shouldSetOrganizationRegisterNumber() {
			fachstelleRegistrationController.preregister(model, authentication);

			verify(model).addAttribute("organizationRegisterNumber", REGISTER_NUMBER);
		}

		@Test
		void shouldSetOrganizationEmail() {
			fachstelleRegistrationController.preregister(model, authentication);

			verify(model).addAttribute("organizationEmail", EMAIL_ADDRESS);
		}

		@Test
		void shouldSetOrganizationAddress() {
			fachstelleRegistrationController.preregister(model, authentication);

			verify(model).addAttribute("organizationAddress", ADDRESS);
		}

		@Test
		void shouldSetRegistration() {
			fachstelleRegistrationController.preregister(model, authentication);

			verify(model).addAttribute(eq("registration"), any(Registration.class));
		}

		@Test
		void shouldSetRegistrationKey() {
			fachstelleRegistrationController.preregister(model, authentication);

			verify(fachstelleRegistrationController).addRegistrationKey(any(User.class));
		}

	}

	@Nested
	class TestSettingRegistrationKey {

		User user = create();

		@Test
		void shouldSetRegistrationKey() {
			fachstelleRegistrationController.addRegistrationKey(user);

			assertThat(user.getRegistrationKey()).isNotNull();
		}

		@Test
		void shouldSetRegistrationExpirationDate() {
			fachstelleRegistrationController.addRegistrationKey(user);

			assertThat(user.getRegistrationKeyExpiresAt()).isBetween(Instant.now().plus(598, ChronoUnit.SECONDS),
			  Instant.now().plus(602, ChronoUnit.SECONDS));
		}

	}

	@Nested
	class TestSuccessPage {

		@Mock
		private Model model;

		@Mock
		private Authentication authentication;

		@Mock
		private HttpServletRequest request;

		@Mock
		private HttpSession session;

		@BeforeEach
		void setUp() {
			when(authentication.getName()).thenReturn("user");
			when(userDetailService.loadUserByUsername("user")).thenReturn(create());
			when(request.getSession()).thenReturn(session);
		}

		@Test
		void shouldCreateSuccessPage() {
			var res = fachstelleRegistrationController.success(model, authentication, request);

			assertThat(res).isEqualTo("success");
		}

		@Test
		void shouldSetUserName() {
			fachstelleRegistrationController.success(model, authentication, request);

			verify(model).addAttribute(eq("name"), any(User.class));
		}

		@Test
		void shouldInvalidateSession() {
			fachstelleRegistrationController.success(model, authentication, request);

			verify(session).invalidate();
		}

		@Test
		void shouldLogout() {
			fachstelleRegistrationController.success(model, authentication, request);

			verify(userDetailService).logout(any(User.class));
		}

	}

	@Nested
	class TestRegistration {

		@Mock
		private Model model;

		@Mock
		private Authentication authentication;

		@Mock
		private Registration registration;

		@BeforeEach
		void setUp() {
			when(authentication.getName()).thenReturn("user");
			var user = create();
			user.setRegistrationKey("key");
			user.setRegistrationKeyExpiresAt(Instant.now().plus(10, ChronoUnit.MINUTES));
			when(userDetailService.loadUserByUsername("user")).thenReturn(user);
			when(registration.key()).thenReturn("key");
		}

		@Test
		void shouldRegister() {
			fachstelleRegistrationController.register(registration, model, authentication);

			verify(registrationService).register(any(User.class));
		}

		@Test
		void shouldReturnSuccessOnRegistration() {
			when(registrationService.register(any(User.class))).thenReturn(true);

			var res = fachstelleRegistrationController.register(registration, model, authentication);

			assertThat(res).isEqualTo("success");
		}

		@Test
		void shouldNotRegister() {
			when(registration.key()).thenReturn("otherKey");
			var res = fachstelleRegistrationController.register(registration, model, authentication);

			assertThat(res).isEqualTo("error");
		}

		@Test
		void shouldSetErrorOnRegistrationKeyError() {
			when(registration.key()).thenReturn("otherKey");

			fachstelleRegistrationController.register(registration, model, authentication);

			verify(model).addAttribute("errorMessageKey", "error_registration_key_does_not_exist");
		}

		@Test
		void shouldSetErrorOnRegistrationError() {
			when(registrationService.register(any(User.class))).thenReturn(false);

			var res = fachstelleRegistrationController.register(registration, model, authentication);

			assertThat(res).isEqualTo("error");
		}

	}

	@Nested
	class TestRegistrationTimeout {

		@Mock
		private Model model;

		@Mock
		private Authentication authentication;

		@Mock
		private Registration registration;

		@BeforeEach
		void setUp() {
			when(authentication.getName()).thenReturn("user");
			var user = create();
			user.setRegistrationKey("key");
			user.setRegistrationKeyExpiresAt(Instant.now().minus(10, ChronoUnit.MINUTES));
			when(userDetailService.loadUserByUsername("user")).thenReturn(user);
			when(registration.key()).thenReturn("key");
		}

		@Test
		void shouldSetErrorOnRegistrationExpired() {
			fachstelleRegistrationController.register(registration, model, authentication);

			verify(model).addAttribute("errorMessageKey", "error_registration_key_expired");
		}

	}

	@Nested
	class TestCanRegister {

		User user;

		@BeforeEach
		void setUp() {
			user = create();
			user.setRegistrationKey("key");
			user.setRegistrationKeyExpiresAt(Instant.now().plus(10, ChronoUnit.MINUTES));
		}

		@Test
		void shouldReturnOk() {
			var res = fachstelleRegistrationController.canRegister(user, "key");

			assertThat(res).isEqualTo("ok");
		}

		@Test
		void shouldReturnExpiredErrorKey() {
			user.setRegistrationKeyExpiresAt(Instant.now().minus(10, ChronoUnit.MINUTES));

			var res = fachstelleRegistrationController.canRegister(user, "key");

			assertThat(res).isEqualTo("error_registration_key_expired");
		}

		@Test
		void shouldReturnNotFoundErrorKey() {
			var res = fachstelleRegistrationController.canRegister(user, "otherHey");

			assertThat(res).isEqualTo("error_registration_key_does_not_exist");
		}

	}

}