/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.fachstelle.resource;

import static de.ozgcloud.fachstelle.resource.OzgcloudResourceController.*;
import static de.ozgcloud.fachstelle.vorgang.VorgangController.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;

import org.hamcrest.core.StringEndsWith;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.TestPropertySource;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;

import de.ozgcloud.fachstelle.vorgang.VorgangTestFactory;
import de.ozgcloud.fachstelle.vorgang.VorgangWithCollaborationManagerAddressTestFactory;
import lombok.SneakyThrows;

@RunWith(SpringRunner.class)
@SpringBootTest
@AutoConfigureMockMvc(addFilters = false, printOnlyOnFailure = false)
@TestPropertySource(properties = {
        "ozgcloud.fachstelle.logout-success-url=http://logout",
        "ozgcloud.fachstelle.login-redirect-url=http://login",
        "ozgcloud.fachstelle.cors=http://login;http://saml-idp",
        "ozgcloud.fachstellen-proxy.base-url=http://proxy",
        "keycloak.auth-server-url=http://keycloak",
        "keycloak.realm=fachstelle",
        "spring.security.saml2.relyingparty.registration.muk.entity-id=http://mock-idp",
        "spring.security.saml2.relyingparty.registration.muk.signing.credentials[0].private-key-location=classpath:/mujina-test.key",
        "spring.security.saml2.relyingparty.registration.muk.signing.credentials[0].certificate-location=classpath:/mujina-test.crt",
        "spring.security.saml2.relyingparty.registration.muk.decryption.credentials[0].private-key-location=classpath:/mujina-test.key",
        "spring.security.saml2.relyingparty.registration.muk.decryption.credentials[0].certificate-location=classpath:/mujina-test.crt",
        "spring.security.saml2.relyingparty.registration.muk.assertingparty.singlesignon.sign-request=false",
        "spring.security.saml2.relyingparty.registration.muk.assertingparty.metadata-uri=classpath:/metadata.xml"
})
@Disabled("need to refactor: The keyfile mujina-test.key is empty")
class OzgcloudResourceControllerITCase {

    @Autowired
    private MockMvc mockMvc;

    @Nested
    class TestGetOzgcloudResource {

        private static final String COLLABORATION_MANAGER_ADDRESS = VorgangWithCollaborationManagerAddressTestFactory.COLLABORATION_MANAGER_ADDRESS;
        private static final String VORGANG_ID = VorgangTestFactory.ID;
        private static final String VALID_URI = COLLABORATION_MANAGER_ADDRESS + "/vorgangs/" + VORGANG_ID;
        private static final String INVALID_URI = COLLABORATION_MANAGER_ADDRESS + ":123/" + VORGANG_ID;

        @Test
        void shouldReturnStatusOk() throws Exception {
            var response = doRequest(VALID_URI);

            response.andExpect(status().isOk());
        }

        @Test
        void shouldHaveVorgangLink() throws Exception {
            var response = doRequest(VALID_URI);

            response.andExpect(jsonPath("$._links.vorgang.href").value(StringEndsWith.endsWith(
                    "/api/vorgangs/" + VORGANG_ID + "?" + PARAM_COLLABORATION_MANAGER_ADDRESS + "=" + COLLABORATION_MANAGER_ADDRESS)));
        }

        @Test
        void shouldHaveSelfLink() throws Exception {
            var encodedUri = URLEncoder.encode(VALID_URI, StandardCharsets.UTF_8);

            var response = doRequest(VALID_URI);

            response.andExpect(
                    jsonPath("$._links.self.href").value(StringEndsWith.endsWith(OzgcloudResourceController.PATH + "?" + PARAM_URI + "=" + encodedUri)));
        }

        @Test
        void shouldReturnStatusNotFound() throws Exception {
            var response = doRequest(INVALID_URI);

            response.andExpect(status().isNotFound());
        }

        @Test
        void shouldReturnBadRequestOnNoRequestParam() throws Exception {
            var response = doRequestWithQueryString("");

            response.andExpect(status().isBadRequest());
        }

        @Test
        void shouldReturnBadRequestOnEmptyUri() throws Exception {
            var response = doRequestWithQueryString("?" + PARAM_URI + "=");

            response.andExpect(status().isBadRequest());
        }

        @SneakyThrows
        private ResultActions doRequest(String uri) {
            return doRequestWithQueryString("?" + PARAM_URI + "=" + uri);
        }

        @SneakyThrows
        private ResultActions doRequestWithQueryString(String queryString) {
            return mockMvc.perform(get(OzgcloudResourceController.PATH + queryString));
        }

    }

}
