/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.fachstelle.security;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.security.authentication.AnonymousAuthenticationToken;
import org.springframework.security.authentication.AuthenticationTrustResolver;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.saml2.provider.service.authentication.Saml2Authentication;

import de.ozgcloud.fachstelle.common.errorhandling.SamlTokenNotFoundException;

@ExtendWith(MockitoExtension.class)
class CurrentUserServiceTest {

	@Spy
	@InjectMocks
	private CurrentUserService service;

	@Mock
	private AuthenticationTrustResolver trustResolver;

	@Nested
	class TestGetAuthentication {

		@Test
		void shouldReturnAuthentication() {
			try (var securityContextHolder = mockStatic(SecurityContextHolder.class)) {
				var authentication = mock(Saml2Authentication.class);
				var context = mock(SecurityContext.class);
				when(context.getAuthentication()).thenReturn(authentication);
				securityContextHolder.when(SecurityContextHolder::getContext).thenReturn(context);

				assertThat(service.getAuthentication()).isEqualTo(authentication);
			}
		}

		@Test
		void shouldThrowExceptionOnUntrustedAuthentication() {
			try (var securityContextHolder = mockStatic(SecurityContextHolder.class)) {
				var authentication = mock(AnonymousAuthenticationToken.class);
				var context = mock(SecurityContext.class);
				when(trustResolver.isAnonymous(authentication)).thenReturn(true);
				when(context.getAuthentication()).thenReturn(authentication);
				securityContextHolder.when(SecurityContextHolder::getContext).thenReturn(context);

				assertThatExceptionOfType(IllegalStateException.class).isThrownBy(service::getAuthentication);
			}
		}

		@Test
		void shouldThrowExceptionOnNullAuthentication() {
			try (var securityContextHolder = mockStatic(SecurityContextHolder.class)) {
				var context = mock(SecurityContext.class);
				when(context.getAuthentication()).thenReturn(null);
				securityContextHolder.when(SecurityContextHolder::getContext).thenReturn(context);

				assertThatExceptionOfType(IllegalStateException.class).isThrownBy(service::getAuthentication);
			}
		}

	}

	@Nested
	class TestGetSamlToken {

		@Test
		void shouldReturnSamlToken() {
			var authentication = mock(Saml2Authentication.class);
			when(authentication.getSaml2Response()).thenReturn(UserTestFactory.SAML_TOKEN);
			doReturn(authentication).when(service).getAuthentication();

			assertThat(service.getSamlToken()).isEqualTo(UserTestFactory.SAML_TOKEN);
		}

		@Test
		void shouldThrowExceptionOnNonSamlAuthentication() {
			var authentication = mock(AnonymousAuthenticationToken.class);
			doReturn(authentication).when(service).getAuthentication();

			assertThatExceptionOfType(SamlTokenNotFoundException.class).isThrownBy(service::getSamlToken);
		}

	}

}