/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.fachstelle.security;

import static org.assertj.core.api.Assertions.*;

import java.util.UUID;
import java.util.concurrent.TimeUnit;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.junit.jupiter.MockitoExtension;

import com.google.common.testing.FakeTicker;

@ExtendWith(MockitoExtension.class)
class InMemoryUserDetailServiceTest {

	private final InMemoryUserDetailService userDetailService = new InMemoryUserDetailService();
	private User user;

	@Nested
	class TestAddingUser {

		@BeforeEach
		void setup() {
			user = UserTestFactory.create();
			userDetailService.addUser(user);
		}

		@Test
		void shouldAddUser() {
			assertThat(userDetailService.getUser(UserTestFactory.USER_ID)).isNotNull();
		}

		@Test
		void shouldLoadUser() {
			userDetailService.addUser(user);

			assertThat(userDetailService.loadUserByUsername(UserTestFactory.USER_NAME)).isNotNull();
		}

	}

	@Nested
	class TestSetUser {

		@BeforeEach
		void setup() {
			user = UserTestFactory.create();
			userDetailService.setUser(user);
		}

		@Test
		void shouldSetUser() {
			assertThat(userDetailService.getUser(UserTestFactory.USER_ID)).isNotNull();
		}

	}

	@Nested
	class TestExpiringCodeCache {

		FakeTicker ticker = new FakeTicker();

		@BeforeEach
		void setup() {
			user = UserTestFactory.create();

			userDetailService.addUser(user);

			ticker.advance(2, TimeUnit.SECONDS);
		}

	}

	@Nested
	class TestUserMapCleanUp {

		private User expiredUser;
		private User validUser;

		@BeforeEach
		void setup() {
			validUser = UserTestFactory.create();
			userDetailService.setUser(validUser);
			expiredUser = UserTestFactory.createBuilder()
			  .id(UUID.randomUUID().toString())
			  .build();
		}

		@Test
		void shouldRemoveExpiredUser() {
			userDetailService.userCleanUp();

			assertThat(userDetailService.getUser(expiredUser.getId())).isNull();
		}

		@Test
		void shouldNotRemoveValidUser() {
			userDetailService.userCleanUp();

			assertThat(userDetailService.getUser(validUser.getId())).isNotNull();
		}

		@Test
		void shouldRemoveOnLogout() {
			userDetailService.logout(validUser);

			assertThat(userDetailService.getUser(validUser.getId())).isNull();
		}

	}

}