/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.fachstelle.security;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.io.File;
import java.io.IOException;
import java.nio.charset.Charset;

import org.apache.commons.io.FileUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.ArgumentCaptor;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.opensaml.core.config.ConfigurationService;
import org.opensaml.core.xml.config.XMLObjectProviderRegistry;
import org.opensaml.core.xml.io.UnmarshallerFactory;
import org.opensaml.core.xml.io.UnmarshallingException;
import org.opensaml.saml.saml2.core.Response;
import org.opensaml.saml.saml2.core.impl.ResponseUnmarshaller;
import org.w3c.dom.Element;

import net.shibboleth.utilities.java.support.component.ComponentInitializationException;

@ExtendWith(MockitoExtension.class)
class Saml2ParserTest {

	@Nested
	class TestGetParserPool {

		@Test
		void shouldHaveParserPool() throws ComponentInitializationException {
			assertThat(Saml2Parser.getParserPool()).isNotNull();
		}

	}

	@Nested
	class TestGetResponseUnmarshaller {

		@Mock
		private UnmarshallerFactory unmarshallerFactory;

		@Mock
		private ResponseUnmarshaller responseUnmarshaller;

		@Mock
		private XMLObjectProviderRegistry providerRegistry;

		@Test
		void shouldHaveResponseUnmarshaller() {
			when(unmarshallerFactory.getUnmarshaller(Response.DEFAULT_ELEMENT_NAME)).thenReturn(responseUnmarshaller);
			when(providerRegistry.getUnmarshallerFactory()).thenReturn(unmarshallerFactory);

			try (var configService = mockStatic(ConfigurationService.class)) {
				configService.when(() -> ConfigurationService.get(XMLObjectProviderRegistry.class)).thenReturn(providerRegistry);

				assertThat(Saml2Parser.getResponseUnmarshaller()).isNotNull();
			}
		}

	}

	@Nested
	class TestParse {

		@Mock
		private UnmarshallerFactory unmarshallerFactory;

		@Mock
		private ResponseUnmarshaller responseUnmarshaller;

		@Mock
		private XMLObjectProviderRegistry providerRegistry;

		private String samlResponse;

		@BeforeEach
		void init() throws IOException, UnmarshallingException {
			samlResponse = FileUtils.readFileToString(new File("src/test/resources/SamlResponse.xml"), Charset.defaultCharset());

			when(responseUnmarshaller.unmarshall(any())).thenReturn(mock(Response.class));
			when(unmarshallerFactory.getUnmarshaller(Response.DEFAULT_ELEMENT_NAME)).thenReturn(responseUnmarshaller);
			when(providerRegistry.getUnmarshallerFactory()).thenReturn(unmarshallerFactory);
		}

		@Test
		void shouldParseSamlToken() {
			try (var configService = mockStatic(ConfigurationService.class)) {
				configService.when(() -> ConfigurationService.get(XMLObjectProviderRegistry.class)).thenReturn(providerRegistry);

				assertThat(Saml2Parser.parse(samlResponse)).isNotNull();
			}
		}

		@Test
		void shouldCreateXmlDocument() throws UnmarshallingException {
			try (var configService = mockStatic(ConfigurationService.class)) {
				configService.when(() -> ConfigurationService.get(XMLObjectProviderRegistry.class)).thenReturn(providerRegistry);

				var xmlElementArgumentCaptor = ArgumentCaptor.forClass(Element.class);

				Saml2Parser.parse(samlResponse);

				verify(responseUnmarshaller).unmarshall(xmlElementArgumentCaptor.capture());
				assertThat(xmlElementArgumentCaptor.getValue().getTagName()).isEqualTo("saml2p:Response");
			}
		}

	}

}