/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.fachstelle.security;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.io.IOException;
import java.util.Collection;
import java.util.List;

import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.servlet.http.HttpSession;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.saml2.provider.service.authentication.Saml2Authentication;

@ExtendWith(MockitoExtension.class)
class UrlAuthenticationSuccessHandlerTest {

	private static final String HTTP_TEST = "http://test";

	private UrlAuthenticationSuccessHandler successHandler;

	@Mock
	private InMemoryUserDetailService userService;

	@Mock
	private UserMapper userMapper;

	@Nested
	class TestOnSuccess {

		UrlAuthenticationSuccessHandler handler;
		@Mock
		private HttpServletRequest request;
		@Mock
		private HttpServletResponse response;
		@Mock
		private Saml2Authentication authentication;

		@BeforeEach
		void setup() {
			handler = spy(new UrlAuthenticationSuccessHandler(userService, userMapper));
			Collection<? extends GrantedAuthority> grantedAuthorities = List.of(new DefaultRole());
			doReturn(grantedAuthorities).when(authentication).getAuthorities();
		}

		@Test
		void shouldCallHandle() throws IOException {
			handler.onAuthenticationSuccess(request, response, authentication);

			verify(handler).handle(any(), any(), any());
		}

		@Test
		void shouldCallClearAuthenticationAttributes() throws IOException {
			handler.onAuthenticationSuccess(request, response, authentication);

			verify(handler).clearAuthenticationAttributes(any());
		}

	}

	@Nested
	class TestWithAuthentication {

		@Mock
		private Authentication authentication;

		@Test
		void shouldDetermineTargetUrlForRole() {
			Collection<? extends GrantedAuthority> grantedAuthorities = List.of(new DefaultRole());
			doReturn(grantedAuthorities).when(authentication).getAuthorities();

			successHandler = new UrlAuthenticationSuccessHandler(userService, userMapper);

			var url = successHandler.determineTargetUrl(authentication);

			assertThat(url).isEqualTo("/preregister");
		}

		@Test
		void shouldDetermineTargetUrlMissingRole() {
			Collection<? extends GrantedAuthority> grantedAuthorities = List.of();
			doReturn(grantedAuthorities).when(authentication).getAuthorities();

			successHandler = new UrlAuthenticationSuccessHandler(userService, userMapper);

			assertThatExceptionOfType(IllegalStateException.class).isThrownBy(() -> successHandler.determineTargetUrl(authentication))
			  .withMessage("Invalid role! User is missing role ROLE_USER");
		}

	}

	@Nested
	class TestClearLoginSession {

		@Mock
		private HttpServletRequest request;

		@Mock
		private HttpSession session;

		@Test
		void shouldClearSession() {
			when(request.getSession(anyBoolean())).thenReturn(session);

			successHandler = new UrlAuthenticationSuccessHandler(userService, userMapper);

			successHandler.clearAuthenticationAttributes(request);

			verify(session).removeAttribute(any());
		}

		@Test
		void shouldHandleNullSession() {
			assertThatNoException().isThrownBy(() -> {
				successHandler = new UrlAuthenticationSuccessHandler(userService, userMapper);
				successHandler.clearAuthenticationAttributes(request);
			});

		}

	}

	@Nested
	class TestHandle {

		@Mock
		private HttpServletRequest request;
		@Mock
		private HttpServletResponse response;
		@Mock
		private Authentication authentication;

		@BeforeEach
		void setup() {
			Collection<? extends GrantedAuthority> grantedAuthorities = List.of(new DefaultRole());
			doReturn(grantedAuthorities).when(authentication).getAuthorities();
		}

		@Test
		void shouldHandle() throws IOException {
			when(request.getContextPath()).thenReturn(HTTP_TEST);
			when(response.isCommitted()).thenReturn(Boolean.FALSE);
			when(response.encodeRedirectURL(anyString())).thenReturn(HTTP_TEST + "/preregister");

			successHandler = new UrlAuthenticationSuccessHandler(userService, userMapper);
			successHandler.handle(request, response, authentication);

			verify(response).encodeRedirectURL(HTTP_TEST + "/preregister");
			verify(response).sendRedirect(startsWith(HTTP_TEST + "/preregister"));
		}

		@Test
		void shouldHandleCommitted() throws IOException {
			when(response.isCommitted()).thenReturn(Boolean.TRUE);
			successHandler = new UrlAuthenticationSuccessHandler(userService, userMapper);
			successHandler.handle(request, response, authentication);

			verify(response, never()).sendRedirect(anyString());
		}

	}

}