/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.fachstelle.security;

import static de.ozgcloud.fachstelle.security.UserAttributeProvider.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.opensaml.core.xml.XMLObject;
import org.opensaml.saml.saml2.core.Attribute;
import org.springframework.security.saml2.Saml2Exception;
import org.springframework.security.saml2.provider.service.authentication.DefaultSaml2AuthenticatedPrincipal;
import org.w3c.dom.Element;

@ExtendWith(MockitoExtension.class)
class UserAttributeProviderTest {

	private static final String UNKNOWN_ATTRIBUTE_KEY = "UnknownAttributeKey";

	@Spy
	@InjectMocks
	private UserAttributeProvider provider;

	@Mock
	private Saml2Decrypter saml2Decrypter;

	private DefaultSaml2AuthenticatedPrincipal principal;

	@BeforeEach
	void setup() {
		Map<String, List<Object>> attributes = new HashMap<>();
		attributes.put(UserAttributeProvider.MUK_FIRMENNAME_KEY, List.of(UserTestFactory.COMPANY_NAME));
		attributes.put(UserAttributeProvider.MUK_RECHTSFORM_KEY, List.of(UserTestFactory.LEGAL_FORM));
		attributes.put(UserAttributeProvider.MUK_RECHTSFORM_TEXT_KEY, List.of(UserTestFactory.LEGAL_FORM_TEXT));
		attributes.put(UserAttributeProvider.MUK_REGISTERNUMMER_KEY, List.of(UserTestFactory.REGISTER_NUMBER));
		attributes.put(UserAttributeProvider.MUK_REGISTERART_KEY, List.of(UserTestFactory.REGISTER_TYPE));
		attributes.put(UserAttributeProvider.MUK_EMAIL_ADRESSE_KEY, List.of(UserTestFactory.EMAIL_ADDRESS));
		attributes.put(UserAttributeProvider.MUK_ADRESSE_KEY, List.of(UserTestFactory.ADDRESS));
		attributes.put(UserAttributeProvider.MUK_VERTRAUENSNIVEAU_KEY, List.of(UserTestFactory.TRUST_LEVEL));
		attributes.put(UNKNOWN_ATTRIBUTE_KEY, List.of(UserTestFactory.USER_ID));

		principal = new DefaultSaml2AuthenticatedPrincipal(UserTestFactory.USER_ID, attributes);
	}

	@Test
	void shouldGetCompanyName() {
		assertThat(provider.getCompanyName(principal)).isEqualTo(UserTestFactory.COMPANY_NAME);
	}

	@Test
	void shouldGetLegalForm() {
		assertThat(provider.getLegalForm(principal)).isEqualTo(UserTestFactory.LEGAL_FORM);
	}

	@Test
	void shouldGetLegalFormText() {
		assertThat(provider.getLegalFormText(principal)).isEqualTo(UserTestFactory.LEGAL_FORM_TEXT);
	}

	@Test
	void shouldGetRegisterNumber() {
		assertThat(provider.getRegisterNumber(principal)).isEqualTo(UserTestFactory.REGISTER_NUMBER);
	}

	@Test
	void shouldGetRegisterType() {
		assertThat(provider.getRegisterType(principal)).isEqualTo(UserTestFactory.REGISTER_TYPE);
	}

	@Test
	void shouldGetEmailAddress() {
		assertThat(provider.getEmailAddress(principal)).isEqualTo(UserTestFactory.EMAIL_ADDRESS);
	}

	@Test
	void shouldGetAddress() {
		var addressNode = mock(Attribute.class);
		var strasseNode = createMockXmlObject(SAML_XML_STRASSE_NODE_NAME, UserTestFactory.STREET);
		var hausnummerNode = createMockXmlObject(SAML_XML_HAUSNUMMER_NODE_NAME, UserTestFactory.HOUSE_NUMBER);
		var plzNode = createMockXmlObject(SAML_XML_PLZ_NODE_NAME, UserTestFactory.POSTAL_CODE);
		var ortNode = createMockXmlObject(SAML_XML_ORT_NODE_NAME, UserTestFactory.CITY);
		var landNode = createMockXmlObject(SAML_XML_LAND_NODE_NAME, UserTestFactory.COUNTRY);
		var attributeValue = mock(XMLObject.class);

		when(attributeValue.getOrderedChildren()).thenReturn(List.of(strasseNode, hausnummerNode, plzNode, ortNode, landNode));
		when(addressNode.getAttributeValues()).thenReturn(List.of(attributeValue));
		when(saml2Decrypter.getDecryptedAttribute(anyString(), anyString())).thenReturn(addressNode);

		assertThat(provider.getAddress("")).isEqualTo(UserTestFactory.ADDRESS);
	}

	@Test
	void shouldHaveNullAddress() {
		var addressNode = mock(Attribute.class);
		var attributeValue = mock(XMLObject.class);

		when(saml2Decrypter.getDecryptedAttribute(anyString(), anyString())).thenReturn(addressNode);
		when(attributeValue.getOrderedChildren()).thenReturn(Collections.emptyList());
		when(addressNode.getAttributeValues()).thenReturn(List.of(attributeValue));

		assertThat(provider.getAddress("")).isNull();
	}

	@Test
	void shouldHaveNullAddressDueToException() {
		when(saml2Decrypter.getDecryptedAttribute(anyString(), anyString())).thenThrow(new Saml2Exception("Decryption error"));

		assertThat(provider.getAddress("")).isNull();
	}

	@Test
	void shouldTrustLevel() {
		assertThat(provider.getTrustLevel(principal)).isEqualTo(UserTestFactory.TRUST_LEVEL);
	}

	@Test
	void shouldGetUnknownAttributes() {
		assertThat(provider.getUnknownAttributes(principal)).hasSize(1);
	}

	private XMLObject createMockXmlObject(String nodeName, String textContent) {
		var node = mock(XMLObject.class);
		var element = mock(Element.class);
		var elementQName = mock(QName.class);
		when(elementQName.getLocalPart()).thenReturn(nodeName);
		when(node.getElementQName()).thenReturn(elementQName);
		when(element.getTextContent()).thenReturn(textContent);
		when(node.getDOM()).thenReturn(element);

		return node;
	}

}