/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.fachstelle.vorgang;

import de.ozgcloud.fachstelle.FachstellenProxyProperties;
import de.ozgcloud.fachstelle.common.errorhandling.SamlTokenNotFoundException;
import de.ozgcloud.fachstelle.proxy.CollaborationGrpcFindVorgangResponse;
import de.ozgcloud.fachstelle.security.CurrentUserService;
import de.ozgcloud.fachstelle.security.UserTestFactory;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.client.RestClient;
import org.springframework.web.client.RestClientException;
import org.springframework.web.util.UriTemplate;

import static de.ozgcloud.fachstelle.vorgang.VorgangRemoteService.FIND_VORGANG_URI;
import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatExceptionOfType;
import static org.mockito.Mockito.*;

@ExtendWith(MockitoExtension.class)
class VorgangRemoteServiceTest {

    @Spy
    @InjectMocks
    private VorgangRemoteService vorgangRemoteService;

    @Mock
    private RestClient fachstellenProxyClient;

    @Mock
    private FachstellenProxyProperties fachstellenProxyProperties;

    @Mock
    private VorgangMapper vorgangMapper;

    @Nested
    class TestFindVorgang {

        private static final String COLLABORATION_MANAGER_ADDRESS_HEADER = "some-header";
        private static final String COLLABORATION_MANAGER_ADDRESS = VorgangWithCollaborationManagerAddressTestFactory.COLLABORATION_MANAGER_ADDRESS;

        private RestClient.RequestHeadersSpec headersSpec;
        private RestClient.ResponseSpec responseSpec;
        private ResponseEntity responseEntity;

        @BeforeEach
        void init() {
            var uriSpec = mock(RestClient.RequestHeadersUriSpec.class);
            headersSpec = mock(RestClient.RequestHeadersSpec.class);
            responseSpec = mock(RestClient.ResponseSpec.class);
            responseEntity = mock(ResponseEntity.class);

            doReturn(FIND_VORGANG_URI).when(vorgangRemoteService).buildFindVorgangUri(VorgangTestFactory.ID);
            when(fachstellenProxyProperties.getCollaborationManagerAddressHeader()).thenReturn(COLLABORATION_MANAGER_ADDRESS_HEADER);
            when(fachstellenProxyClient.get()).thenReturn(uriSpec);
            when(uriSpec.uri(FIND_VORGANG_URI)).thenReturn(headersSpec);
            when(headersSpec.accept(MediaType.APPLICATION_JSON)).thenReturn(headersSpec);
            when(headersSpec.header(COLLABORATION_MANAGER_ADDRESS_HEADER, COLLABORATION_MANAGER_ADDRESS)).thenReturn(headersSpec);
            when(headersSpec.retrieve()).thenReturn(responseSpec);
        }

        @Test
        void shouldCallRestClient() {
            when(responseSpec.toEntity(CollaborationGrpcFindVorgangResponse.class)).thenReturn(responseEntity);
            when(responseEntity.getBody()).thenReturn(new CollaborationGrpcFindVorgangResponse());

            vorgangRemoteService.findVorgang(VorgangTestFactory.ID, COLLABORATION_MANAGER_ADDRESS);

            verify(fachstellenProxyClient).get();
        }

        @Test
        void shouldCallVorgangMapper() {
            when(responseSpec.toEntity(CollaborationGrpcFindVorgangResponse.class)).thenReturn(responseEntity);
            when(responseEntity.getBody()).thenReturn(new CollaborationGrpcFindVorgangResponse());

            vorgangRemoteService.findVorgang(VorgangTestFactory.ID, COLLABORATION_MANAGER_ADDRESS);

            verify(vorgangMapper).toVorgang(any());
        }

        @Test
        void shouldGetResponse() {
            when(responseSpec.toEntity(CollaborationGrpcFindVorgangResponse.class)).thenReturn(responseEntity);
            when(responseEntity.getBody()).thenReturn(new CollaborationGrpcFindVorgangResponse());
            when(vorgangMapper.toVorgang(any())).thenReturn(VorgangTestFactory.create());

            var response = vorgangRemoteService.findVorgang(VorgangTestFactory.ID, COLLABORATION_MANAGER_ADDRESS);

            assertThat(response).isNotNull();
        }

        @Test
        void shouldThrowException() {
            when(headersSpec.retrieve()).thenThrow(new RestClientException("some error"));

            assertThatExceptionOfType(RestClientException.class).isThrownBy(
                    () -> vorgangRemoteService.findVorgang(VorgangTestFactory.ID, COLLABORATION_MANAGER_ADDRESS));
        }

    }

    @Nested
    class TestBuildFindVorgangUri {

        @Test
        void shouldReturnVorgangUri() {
            final String expectedUri = new UriTemplate(FIND_VORGANG_URI).expand(VorgangTestFactory.ID, VorgangRemoteService.DUMMY_SAML_TOKEN).toString();
            final String uri = vorgangRemoteService.buildFindVorgangUri(VorgangTestFactory.ID);

            assertThat(uri).isEqualTo(expectedUri);
        }
    }

}
