/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.fachstelle.vorgang;

import static de.ozgcloud.fachstelle.vorgang.VorgangController.*;
import static org.assertj.core.api.Assertions.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.MethodSource;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.LinkRelation;

class VorgangURIResolverTest {

	private final VorgangURIResolver resolver = new VorgangURIResolver();

	@Nested
	class TestResolve {

		private static final String VORGANG_ID = VorgangTestFactory.ID;

		@ParameterizedTest
		@MethodSource("validVorgangUris")
		void shouldReturnLink(String uri) {
			var result = resolver.resolve(uri);

			assertThat(result).isPresent();
		}

		@ParameterizedTest
		@MethodSource("invalidVorgangUris")
		void shouldReturnEmptyIfUriIsNotRecognized(String uri) {
			var result = resolver.resolve(uri);

			assertThat(result).isEmpty();
		}

		@Nested
		class TestVorgangLink {

			private static final String COLLABORATION_MANAGER_ADDRESS = VorgangWithCollaborationManagerAddressTestFactory.COLLABORATION_MANAGER_ADDRESS;
			private static final String URI = COLLABORATION_MANAGER_ADDRESS + "/vorgangs/" + VORGANG_ID;

			@Test
			void shouldHaveCorrectRelValue() {
				var result = resolver.resolve(URI);

				assertThat(result).get().extracting(Link::getRel).extracting(LinkRelation::value).isEqualTo(VorgangURIResolver.REL_NAME);
			}

			@Test
			void shouldHaveHrefOfVorgang() {
				var result = resolver.resolve(URI);

				assertThat(result).get().extracting(Link::getHref)
				  .isEqualTo("/api/vorgangs/" + VORGANG_ID + "?" + PARAM_COLLABORATION_MANAGER_ADDRESS + "=" + COLLABORATION_MANAGER_ADDRESS);
			}

		}

		private static Stream<String> validVorgangUris() {
			return Stream.of("dummy.service/vorgangs/" + VORGANG_ID,
			  "dummy-service.dummy/vorgangs/" + VORGANG_ID,
			  "dummy.service/vorgangs/" + VORGANG_ID);
		}

		private static Stream<String> invalidVorgangUris() {
			return Stream.of("dummy.service:123/vorgangs/" + VORGANG_ID,
			  "dns:///dummy.service/vorgangs/" + VORGANG_ID,
			  "dummy.service/res-a/" + VORGANG_ID,
			  "dummy.service/vorgangs/abc/" + VORGANG_ID,
			  "dummy.service/vorgangs/" + VORGANG_ID + "/" + VORGANG_ID);
		}

	}

}
